#ifndef dx7ctrl_processParameter_h
#define dx7ctrl_processParameter_h

#include "ftypes.h"

namespace Steinberg {
    namespace Vst {
        
        class ProcessParameter {
        public:            
            ProcessParameter(uchar messageID, uchar pType, float defValue = 0.): value(defValue) {
                byteRep[0] = messageID;
                byteRep[1] = pType;
            }
            
            float getValue() {
                return value;
            }            
            void update(float newValue) {
                value = newValue;
                updateByteValue(newValue);
            }
            
            uchar* getByteRep(void) {
                return byteRep;
            }
        protected:
            virtual void updateByteValue(float value) = 0;
            float value;
            uchar byteRep[6];        
        };
        
        
        class ProcessContinuousParameter : public ProcessParameter {
        public:
            ProcessContinuousParameter(uchar messageID, uchar pType, float defValue = 0.): ProcessParameter(messageID, pType, defValue) {
                updateByteValue(defValue);
            }
        protected:
            virtual void updateByteValue(float value) { 
                uint32 intValue = value < 1u? (uint32) (value * 4294967295u) : 4294967295u;
                byteRep[2] = (uchar) intValue;
                byteRep[3] = (uchar) (intValue >> 8);
                byteRep[4] = (uchar) (intValue >> 16);
                byteRep[5] = (uchar) (intValue >> 24);
            }
        };

        
        class ProcessDiscreteParameter : public ProcessParameter {
        public:
            ProcessDiscreteParameter(uchar messageID, uchar pType, float defValue = 0.5, uint32 stepCount = 1): ProcessParameter(messageID, pType, defValue), stepCount(stepCount) {
                updateByteValue(defValue);
            }
        protected:
            virtual void updateByteValue(float value) {
                uint32 temp = (uint32) (value * (stepCount+1));
                uint32 intValue = temp > stepCount? stepCount : temp;
                byteRep[2] = (uchar) intValue;
                byteRep[3] = (uchar) (intValue >> 8);
                byteRep[4] = (uchar) (intValue >> 16);
                byteRep[5] = (uchar) (intValue >> 24);
            }
            uint32 stepCount;            
        };
        
        
        class ProcessCoarseParameter : public ProcessParameter {
        public:
            ProcessCoarseParameter(uchar messageID, uchar pType, float defValue = 0.5): ProcessParameter(messageID, pType, defValue) {
                updateByteValue(defValue);
            }
        protected:
            virtual void updateByteValue(float value) {
                uchar ratio = (uchar) (value * 32);
                ratio = ratio > 31? 31 : ratio;
                ratio = ratio > 0? ratio << 1 : 1;
                
                byteRep[2] = ratio;
                byteRep[3] = 0;
                byteRep[4] = 0;
                byteRep[5] = 0;
            }         
        };
        
        
        
        class ProcessFreqParameter : public ProcessParameter {
        public:
            ProcessFreqParameter(uchar messageID, uchar pType, float defValue = 0.04390439043904390439043904390439): ProcessParameter(messageID, pType, defValue) {
                updateByteValue(defValue);
            }
        protected:
            virtual void updateByteValue(float value) { 
                uint32 intValue = value < 1u? (uint32) (value * 1000000u) : 1000000u;
                byteRep[2] = (uchar) intValue;
                byteRep[3] = (uchar) (intValue >> 8);
                byteRep[4] = (uchar) (intValue >> 16);
                byteRep[5] = (uchar) (intValue >> 24);
            }
        };

    }
}



#endif
